@@ -269,8 +269,8 @@ func TestDataSourceFilter(t *testing.T) {
 
 	for testName, test := range tests {
 		t.Run(testName, func(t *testing.T) {
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.AnyVolumeDataSource, test.anyEnabled)()
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CrossNamespaceVolumeDataSource, test.xnsEnabled)()
+			featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.AnyVolumeDataSource, test.anyEnabled)
+			featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CrossNamespaceVolumeDataSource, test.xnsEnabled)
 			DropDisabledFields(&test.spec, &test.oldSpec)
 			if test.spec.DataSource != test.want {
 				t.Errorf("expected condition was not met, test: %s, anyEnabled: %v, xnsEnabled: %v, spec: %+v, expected DataSource: %+v",
@@ -367,8 +367,8 @@ func TestDataSourceRef(t *testing.T) {
 		},
 	}
 
-	defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.AnyVolumeDataSource, true)()
-	defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CrossNamespaceVolumeDataSource, true)()
+	featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.AnyVolumeDataSource, true)
+	featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CrossNamespaceVolumeDataSource, true)
 
 	for testName, test := range tests {
 		t.Run(testName, func(t *testing.T) {
@@ -435,7 +435,7 @@ func TestDropDisabledVolumeAttributesClass(t *testing.T) {
 
 	for testName, test := range tests {
 		t.Run(testName, func(t *testing.T) {
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.VolumeAttributesClass, test.vacEnabled)()
+			featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.VolumeAttributesClass, test.vacEnabled)
 			DropDisabledFields(&test.spec, &test.oldSpec)
 			if test.spec.VolumeAttributesClassName != test.wantVAC {
 				t.Errorf("expected vac was not met, test: %s, vacEnabled: %v, spec: %+v, expected VAC: %+v",
@@ -498,7 +498,7 @@ func TestDropDisabledFieldsFromStatus(t *testing.T) {
 			name:                                "for:newPVC=hasResizeStatus,oldPVC=nil, featuregate=false should drop field",
 			enableRecoverVolumeExpansionFailure: false,
 			enableVolumeAttributesClass:         false,
-			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
+			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
 			oldPVC:                              nil,
 			expected:                            getPVC(),
 		},
@@ -506,25 +506,25 @@ func TestDropDisabledFieldsFromStatus(t *testing.T) {
 			name:                                "for:newPVC=hasResizeStatus,oldPVC=doesnot,featuregate=RecoverVolumeExpansionFailure=true; should keep field",
 			enableRecoverVolumeExpansionFailure: true,
 			enableVolumeAttributesClass:         false,
-			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
+			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
 			oldPVC:                              getPVC(),
-			expected:                            withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
+			expected:                            withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
 		},
 		{
 			name:                                "for:newPVC=hasResizeStatus,oldPVC=hasResizeStatus,featuregate=RecoverVolumeExpansionFailure=true; should keep field",
 			enableRecoverVolumeExpansionFailure: true,
 			enableVolumeAttributesClass:         false,
-			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
-			oldPVC:                              withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
-			expected:                            withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
+			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
+			oldPVC:                              withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
+			expected:                            withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
 		},
 		{
 			name:                                "for:newPVC=hasResizeStatus,oldPVC=hasResizeStatus,featuregate=false; should keep field",
 			enableRecoverVolumeExpansionFailure: false,
 			enableVolumeAttributesClass:         false,
-			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
-			oldPVC:                              withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
-			expected:                            withResizeStatus(core.PersistentVolumeClaimNodeResizeFailed),
+			pvc:                                 withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
+			oldPVC:                              withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
+			expected:                            withResizeStatus(core.PersistentVolumeClaimNodeResizeInfeasible),
 		},
 		{
 			name:                                "for:newPVC=hasVolumeAttributeClass,oldPVC=nil, featuregate=false should drop field",
@@ -594,8 +594,8 @@ func TestDropDisabledFieldsFromStatus(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.RecoverVolumeExpansionFailure, test.enableRecoverVolumeExpansionFailure)()
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.VolumeAttributesClass, test.enableVolumeAttributesClass)()
+			featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.RecoverVolumeExpansionFailure, test.enableRecoverVolumeExpansionFailure)
+			featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.VolumeAttributesClass, test.enableVolumeAttributesClass)
 
 			DropDisabledFieldsFromStatus(test.pvc, test.oldPVC)
 
