@@ -19,8 +19,6 @@ package validation
 import (
 	"testing"
 
-	"github.com/stretchr/testify/assert"
-
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/validation/field"
 	"k8s.io/kubernetes/pkg/apis/resource"
@@ -192,7 +190,7 @@ func TestValidateClaimTemplate(t *testing.T) {
 	for name, scenario := range scenarios {
 		t.Run(name, func(t *testing.T) {
 			errs := ValidateResourceClaimTemplate(scenario.template)
-			assert.Equal(t, scenario.wantFailures, errs)
+			assertFailures(t, scenario.wantFailures, errs)
 		})
 	}
 }
@@ -227,7 +225,7 @@ func TestValidateClaimTemplateUpdate(t *testing.T) {
 		t.Run(name, func(t *testing.T) {
 			scenario.oldClaimTemplate.ResourceVersion = "1"
 			errs := ValidateResourceClaimTemplateUpdate(scenario.update(scenario.oldClaimTemplate.DeepCopy()), scenario.oldClaimTemplate)
-			assert.Equal(t, scenario.wantFailures, errs)
+			assertFailures(t, scenario.wantFailures, errs)
 		})
 	}
 }
