@@ -18,9 +18,7 @@ package cm
 
 import (
 	"k8s.io/api/core/v1"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
-	runtimeapi "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
-	kubefeatures "k8s.io/kubernetes/pkg/features"
+	runtimeapi "k8s.io/cri-api/pkg/apis/runtime/v1"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpumanager"
 	"k8s.io/kubernetes/pkg/kubelet/cm/memorymanager"
 	"k8s.io/kubernetes/pkg/kubelet/cm/topologymanager"
@@ -29,7 +27,6 @@ import (
 type InternalContainerLifecycle interface {
 	PreCreateContainer(pod *v1.Pod, container *v1.Container, containerConfig *runtimeapi.ContainerConfig) error
 	PreStartContainer(pod *v1.Pod, container *v1.Container, containerID string) error
-	PreStopContainer(containerID string) error
 	PostStopContainer(containerID string) error
 }
 
@@ -49,22 +46,11 @@ func (i *internalContainerLifecycleImpl) PreStartContainer(pod *v1.Pod, containe
 		i.memoryManager.AddContainer(pod, container, containerID)
 	}
 
-	if utilfeature.DefaultFeatureGate.Enabled(kubefeatures.TopologyManager) {
-		i.topologyManager.AddContainer(pod, container, containerID)
-	}
-	return nil
-}
+	i.topologyManager.AddContainer(pod, container, containerID)
 
-func (i *internalContainerLifecycleImpl) PreStopContainer(containerID string) error {
 	return nil
 }
 
 func (i *internalContainerLifecycleImpl) PostStopContainer(containerID string) error {
-	if utilfeature.DefaultFeatureGate.Enabled(kubefeatures.TopologyManager) {
-		err := i.topologyManager.RemoveContainer(containerID)
-		if err != nil {
-			return err
-		}
-	}
-	return nil
+	return i.topologyManager.RemoveContainer(containerID)
 }
