@@ -47,9 +47,6 @@ func (flowSchemaStrategy) NamespaceScoped() bool {
 // and should not be modified by the user.
 func (flowSchemaStrategy) GetResetFields() map[fieldpath.APIVersion]*fieldpath.Set {
 	fields := map[fieldpath.APIVersion]*fieldpath.Set{
-		"flowcontrol.apiserver.k8s.io/v1alpha1": fieldpath.NewSet(
-			fieldpath.MakePathOrDie("status"),
-		),
 		"flowcontrol.apiserver.k8s.io/v1beta1": fieldpath.NewSet(
 			fieldpath.MakePathOrDie("status"),
 		),
@@ -59,6 +56,9 @@ func (flowSchemaStrategy) GetResetFields() map[fieldpath.APIVersion]*fieldpath.S
 		"flowcontrol.apiserver.k8s.io/v1beta3": fieldpath.NewSet(
 			fieldpath.MakePathOrDie("status"),
 		),
+		"flowcontrol.apiserver.k8s.io/v1": fieldpath.NewSet(
+			fieldpath.MakePathOrDie("status"),
+		),
 	}
 
 	return fields
@@ -127,10 +127,6 @@ var StatusStrategy = flowSchemaStatusStrategy{Strategy}
 // and should not be modified by the user.
 func (flowSchemaStatusStrategy) GetResetFields() map[fieldpath.APIVersion]*fieldpath.Set {
 	fields := map[fieldpath.APIVersion]*fieldpath.Set{
-		"flowcontrol.apiserver.k8s.io/v1alpha1": fieldpath.NewSet(
-			fieldpath.MakePathOrDie("metadata"),
-			fieldpath.MakePathOrDie("spec"),
-		),
 		"flowcontrol.apiserver.k8s.io/v1beta1": fieldpath.NewSet(
 			fieldpath.MakePathOrDie("metadata"),
 			fieldpath.MakePathOrDie("spec"),
@@ -143,6 +139,10 @@ func (flowSchemaStatusStrategy) GetResetFields() map[fieldpath.APIVersion]*field
 			fieldpath.MakePathOrDie("metadata"),
 			fieldpath.MakePathOrDie("spec"),
 		),
+		"flowcontrol.apiserver.k8s.io/v1": fieldpath.NewSet(
+			fieldpath.MakePathOrDie("metadata"),
+			fieldpath.MakePathOrDie("spec"),
+		),
 	}
 
 	return fields
