@@ -18,11 +18,13 @@ package testing
 
 import (
 	"fmt"
+	"slices"
 	"time"
 
 	v1 "k8s.io/api/core/v1"
 	policy "k8s.io/api/policy/v1"
-	resourceapi "k8s.io/api/resource/v1alpha3"
+	resourceapi "k8s.io/api/resource/v1beta1"
+	storagev1 "k8s.io/api/storage/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime/schema"
@@ -42,17 +44,28 @@ func MakeNodeSelector() *NodeSelectorWrapper {
 	return &NodeSelectorWrapper{v1.NodeSelector{}}
 }
 
+type NodeSelectorType int
+
+const (
+	NodeSelectorTypeMatchExpressions NodeSelectorType = iota
+	NodeSelectorTypeMatchFields
+)
+
 // In injects a matchExpression (with an operator IN) as a selectorTerm
 // to the inner nodeSelector.
 // NOTE: appended selecterTerms are ORed.
-func (s *NodeSelectorWrapper) In(key string, vals []string) *NodeSelectorWrapper {
+func (s *NodeSelectorWrapper) In(key string, vals []string, t NodeSelectorType) *NodeSelectorWrapper {
 	expression := v1.NodeSelectorRequirement{
 		Key:      key,
 		Operator: v1.NodeSelectorOpIn,
 		Values:   vals,
 	}
 	selectorTerm := v1.NodeSelectorTerm{}
-	selectorTerm.MatchExpressions = append(selectorTerm.MatchExpressions, expression)
+	if t == NodeSelectorTypeMatchExpressions {
+		selectorTerm.MatchExpressions = append(selectorTerm.MatchExpressions, expression)
+	} else {
+		selectorTerm.MatchFields = append(selectorTerm.MatchFields, expression)
+	}
 	s.NodeSelectorTerms = append(s.NodeSelectorTerms, selectorTerm)
 	return s
 }
@@ -214,6 +227,12 @@ func (c *ContainerWrapper) ResourceLimits(limMap map[v1.ResourceName]string) *Co
 	return c
 }
 
+// RestartPolicy sets the container's restartPolicy to the given restartPolicy.
+func (c *ContainerWrapper) RestartPolicy(restartPolicy v1.ContainerRestartPolicy) *ContainerWrapper {
+	c.Container.RestartPolicy = &restartPolicy
+	return c
+}
+
 // PodDisruptionBudgetWrapper wraps a PodDisruptionBudget inside.
 type PodDisruptionBudgetWrapper struct {
 	policy.PodDisruptionBudget
@@ -291,6 +310,12 @@ func (p *PodWrapper) Name(s string) *PodWrapper {
 	return p
 }
 
+// Name sets `s` as the name of the inner pod.
+func (p *PodWrapper) GenerateName(s string) *PodWrapper {
+	p.SetGenerateName(s)
+	return p
+}
+
 // UID sets `s` as the UID of the inner pod.
 func (p *PodWrapper) UID(s string) *PodWrapper {
 	p.SetUID(types.UID(s))
@@ -309,6 +334,12 @@ func (p *PodWrapper) Namespace(s string) *PodWrapper {
 	return p
 }
 
+// Resources sets requests and limits at pod-level.
+func (p *PodWrapper) Resources(resources v1.ResourceRequirements) *PodWrapper {
+	p.Spec.Resources = &resources
+	return p
+}
+
 // OwnerReference updates the owning controller of the pod.
 func (p *PodWrapper) OwnerReference(name string, gvk schema.GroupVersionKind) *PodWrapper {
 	p.OwnerReferences = []metav1.OwnerReference{
@@ -341,6 +372,12 @@ func (p *PodWrapper) PodResourceClaims(podResourceClaims ...v1.PodResourceClaim)
 	return p
 }
 
+// PodResourceClaims appends claim statuses into PodSpec of the inner pod.
+func (p *PodWrapper) ResourceClaimStatuses(resourceClaimStatuses ...v1.PodResourceClaimStatus) *PodWrapper {
+	p.Status.ResourceClaimStatuses = append(p.Status.ResourceClaimStatuses, resourceClaimStatuses...)
+	return p
+}
+
 // Priority sets a priority value into PodSpec of the inner pod.
 func (p *PodWrapper) Priority(val int32) *PodWrapper {
 	p.Spec.Priority = &val
@@ -380,19 +417,19 @@ func (p *PodWrapper) NodeSelector(m map[string]string) *PodWrapper {
 
 // NodeAffinityIn creates a HARD node affinity (with the operator In)
 // and injects into the inner pod.
-func (p *PodWrapper) NodeAffinityIn(key string, vals []string) *PodWrapper {
+func (p *PodWrapper) NodeAffinityIn(key string, vals []string, t NodeSelectorType) *PodWrapper {
 	if p.Spec.Affinity == nil {
 		p.Spec.Affinity = &v1.Affinity{}
 	}
 	if p.Spec.Affinity.NodeAffinity == nil {
 		p.Spec.Affinity.NodeAffinity = &v1.NodeAffinity{}
 	}
-	nodeSelector := MakeNodeSelector().In(key, vals).Obj()
+	nodeSelector := MakeNodeSelector().In(key, vals, t).Obj()
 	p.Spec.Affinity.NodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution = nodeSelector
 	return p
 }
 
-// NodeAffinityNotIn creates a HARD node affinity (with the operator NotIn)
+// NodeAffinityNotIn creates a HARD node affinity (with MatchExpressinos and the operator NotIn)
 // and injects into the inner pod.
 func (p *PodWrapper) NodeAffinityNotIn(key string, vals []string) *PodWrapper {
 	if p.Spec.Affinity == nil {
@@ -743,6 +780,17 @@ func (p *PodWrapper) InitReq(resMap map[v1.ResourceName]string) *PodWrapper {
 	return p
 }
 
+// SidecarReq adds a new sidecar container to the inner pod with given resource map.
+func (p *PodWrapper) SidecarReq(resMap map[v1.ResourceName]string) *PodWrapper {
+	if len(resMap) == 0 {
+		return p
+	}
+
+	name := fmt.Sprintf("sidecar-con%d", len(p.Spec.InitContainers))
+	p.Spec.InitContainers = append(p.Spec.InitContainers, MakeContainer().Name(name).Image(imageutils.GetPauseImageName()).RestartPolicy(v1.ContainerRestartPolicyAlways).Resources(resMap).Obj())
+	return p
+}
+
 // PreemptionPolicy sets the give preemption policy to the inner pod.
 func (p *PodWrapper) PreemptionPolicy(policy v1.PreemptionPolicy) *PodWrapper {
 	p.Spec.PreemptionPolicy = &policy
@@ -903,6 +951,13 @@ func (p *PersistentVolumeClaimWrapper) Resources(resources v1.VolumeResourceRequ
 	return p
 }
 
+// StorageClassName sets `StorageClassName` as the StorageClassName of the inner
+// PersistentVolumeClaim.
+func (p *PersistentVolumeClaimWrapper) StorageClassName(name *string) *PersistentVolumeClaimWrapper {
+	p.PersistentVolumeClaim.Spec.StorageClassName = name
+	return p
+}
+
 // PersistentVolumeWrapper wraps a PersistentVolume inside.
 type PersistentVolumeWrapper struct{ v1.PersistentVolume }
 
@@ -942,7 +997,13 @@ func (p *PersistentVolumeWrapper) HostPathVolumeSource(src *v1.HostPathVolumeSou
 	return p
 }
 
-// NodeAffinityIn creates a HARD node affinity (with the operator In)
+// PersistentVolumeSource sets `src` as the pv source of the inner
+func (p *PersistentVolumeWrapper) PersistentVolumeSource(src v1.PersistentVolumeSource) *PersistentVolumeWrapper {
+	p.PersistentVolume.Spec.PersistentVolumeSource = src
+	return p
+}
+
+// NodeAffinityIn creates a HARD node affinity (with MatchExpressions and the operator In)
 // and injects into the pv.
 func (p *PersistentVolumeWrapper) NodeAffinityIn(key string, vals []string) *PersistentVolumeWrapper {
 	if p.Spec.NodeAffinity == nil {
@@ -951,17 +1012,40 @@ func (p *PersistentVolumeWrapper) NodeAffinityIn(key string, vals []string) *Per
 	if p.Spec.NodeAffinity.Required == nil {
 		p.Spec.NodeAffinity.Required = &v1.NodeSelector{}
 	}
-	nodeSelector := MakeNodeSelector().In(key, vals).Obj()
+	nodeSelector := MakeNodeSelector().In(key, vals, NodeSelectorTypeMatchExpressions).Obj()
 	p.Spec.NodeAffinity.Required.NodeSelectorTerms = append(p.Spec.NodeAffinity.Required.NodeSelectorTerms, nodeSelector.NodeSelectorTerms...)
 	return p
 }
 
+// Labels sets all {k,v} pair provided by `labels` to the pv.
+func (p *PersistentVolumeWrapper) Labels(labels map[string]string) *PersistentVolumeWrapper {
+	for k, v := range labels {
+		p.Label(k, v)
+	}
+	return p
+}
+
+// Label sets a {k,v} pair to the pv.
+func (p *PersistentVolumeWrapper) Label(k, v string) *PersistentVolumeWrapper {
+	if p.PersistentVolume.ObjectMeta.Labels == nil {
+		p.PersistentVolume.ObjectMeta.Labels = make(map[string]string)
+	}
+	p.PersistentVolume.ObjectMeta.Labels[k] = v
+	return p
+}
+
+// StorageClassName sets `StorageClassName` of the inner PersistentVolume.
+func (p *PersistentVolumeWrapper) StorageClassName(name string) *PersistentVolumeWrapper {
+	p.PersistentVolume.Spec.StorageClassName = name
+	return p
+}
+
 // ResourceClaimWrapper wraps a ResourceClaim inside.
 type ResourceClaimWrapper struct{ resourceapi.ResourceClaim }
 
 // MakeResourceClaim creates a ResourceClaim wrapper.
-func MakeResourceClaim(controller string) *ResourceClaimWrapper {
-	return &ResourceClaimWrapper{resourceapi.ResourceClaim{Spec: resourceapi.ResourceClaimSpec{Controller: controller}}}
+func MakeResourceClaim() *ResourceClaimWrapper {
+	return &ResourceClaimWrapper{}
 }
 
 // FromResourceClaim creates a ResourceClaim wrapper from some existing object.
@@ -1022,25 +1106,16 @@ func (wrapper *ResourceClaimWrapper) Request(deviceClassName string) *ResourceCl
 
 // Allocation sets the allocation of the inner object.
 func (wrapper *ResourceClaimWrapper) Allocation(allocation *resourceapi.AllocationResult) *ResourceClaimWrapper {
+	if !slices.Contains(wrapper.ResourceClaim.Finalizers, resourceapi.Finalizer) {
+		wrapper.ResourceClaim.Finalizers = append(wrapper.ResourceClaim.Finalizers, resourceapi.Finalizer)
+	}
 	wrapper.ResourceClaim.Status.Allocation = allocation
 	return wrapper
 }
 
-// Structured turns a "normal" claim into one which was allocated via structured parameters.
-// The only difference is that there is no controller name and the special finalizer
-// gets added.
-func (wrapper *ResourceClaimWrapper) Structured() *ResourceClaimWrapper {
-	wrapper.Spec.Controller = ""
-	if wrapper.ResourceClaim.Status.Allocation != nil {
-		wrapper.ResourceClaim.Finalizers = append(wrapper.ResourceClaim.Finalizers, resourceapi.Finalizer)
-		wrapper.ResourceClaim.Status.Allocation.Controller = ""
-	}
-	return wrapper
-}
-
-// DeallocationRequested sets that field of the inner object.
-func (wrapper *ResourceClaimWrapper) DeallocationRequested(deallocationRequested bool) *ResourceClaimWrapper {
-	wrapper.ResourceClaim.Status.DeallocationRequested = deallocationRequested
+// Deleting sets the deletion timestamp of the inner object.
+func (wrapper *ResourceClaimWrapper) Deleting(time metav1.Time) *ResourceClaimWrapper {
+	wrapper.ResourceClaim.DeletionTimestamp = &time
 	return wrapper
 }
 
@@ -1050,91 +1125,11 @@ func (wrapper *ResourceClaimWrapper) ReservedFor(consumers ...resourceapi.Resour
 	return wrapper
 }
 
-// ReservedFor sets that field of the inner object given information about one pod.
+// ReservedForPod sets that field of the inner object given information about one pod.
 func (wrapper *ResourceClaimWrapper) ReservedForPod(podName string, podUID types.UID) *ResourceClaimWrapper {
 	return wrapper.ReservedFor(resourceapi.ResourceClaimConsumerReference{Resource: "pods", Name: podName, UID: podUID})
 }
 
-// PodSchedulingWrapper wraps a PodSchedulingContext inside.
-type PodSchedulingWrapper struct {
-	resourceapi.PodSchedulingContext
-}
-
-// MakePodSchedulingContexts creates a PodSchedulingContext wrapper.
-func MakePodSchedulingContexts() *PodSchedulingWrapper {
-	return &PodSchedulingWrapper{resourceapi.PodSchedulingContext{}}
-}
-
-// FromPodSchedulingContexts creates a PodSchedulingContext wrapper from an existing object.
-func FromPodSchedulingContexts(other *resourceapi.PodSchedulingContext) *PodSchedulingWrapper {
-	return &PodSchedulingWrapper{*other.DeepCopy()}
-}
-
-// Obj returns the inner object.
-func (wrapper *PodSchedulingWrapper) Obj() *resourceapi.PodSchedulingContext {
-	return &wrapper.PodSchedulingContext
-}
-
-// Name sets `s` as the name of the inner object.
-func (wrapper *PodSchedulingWrapper) Name(s string) *PodSchedulingWrapper {
-	wrapper.SetName(s)
-	return wrapper
-}
-
-// UID sets `s` as the UID of the inner object.
-func (wrapper *PodSchedulingWrapper) UID(s string) *PodSchedulingWrapper {
-	wrapper.SetUID(types.UID(s))
-	return wrapper
-}
-
-// Namespace sets `s` as the namespace of the inner object.
-func (wrapper *PodSchedulingWrapper) Namespace(s string) *PodSchedulingWrapper {
-	wrapper.SetNamespace(s)
-	return wrapper
-}
-
-// OwnerReference updates the owning controller of the inner object.
-func (wrapper *PodSchedulingWrapper) OwnerReference(name, uid string, gvk schema.GroupVersionKind) *PodSchedulingWrapper {
-	wrapper.OwnerReferences = []metav1.OwnerReference{
-		{
-			APIVersion:         gvk.GroupVersion().String(),
-			Kind:               gvk.Kind,
-			Name:               name,
-			UID:                types.UID(uid),
-			Controller:         ptr.To(true),
-			BlockOwnerDeletion: ptr.To(true),
-		},
-	}
-	return wrapper
-}
-
-// Label applies a {k,v} label pair to the inner object
-func (wrapper *PodSchedulingWrapper) Label(k, v string) *PodSchedulingWrapper {
-	if wrapper.Labels == nil {
-		wrapper.Labels = make(map[string]string)
-	}
-	wrapper.Labels[k] = v
-	return wrapper
-}
-
-// SelectedNode sets that field of the inner object.
-func (wrapper *PodSchedulingWrapper) SelectedNode(s string) *PodSchedulingWrapper {
-	wrapper.Spec.SelectedNode = s
-	return wrapper
-}
-
-// PotentialNodes sets that field of the inner object.
-func (wrapper *PodSchedulingWrapper) PotentialNodes(nodes ...string) *PodSchedulingWrapper {
-	wrapper.Spec.PotentialNodes = nodes
-	return wrapper
-}
-
-// ResourceClaims sets that field of the inner object.
-func (wrapper *PodSchedulingWrapper) ResourceClaims(statuses ...resourceapi.ResourceClaimSchedulingStatus) *PodSchedulingWrapper {
-	wrapper.Status.ResourceClaims = statuses
-	return wrapper
-}
-
 type ResourceSliceWrapper struct {
 	resourceapi.ResourceSlice
 }
@@ -1148,10 +1143,16 @@ func MakeResourceSlice(nodeName, driverName string) *ResourceSliceWrapper {
 	return wrapper
 }
 
+// FromResourceSlice creates a ResourceSlice wrapper from some existing object.
+func FromResourceSlice(other *resourceapi.ResourceSlice) *ResourceSliceWrapper {
+	return &ResourceSliceWrapper{*other.DeepCopy()}
+}
+
 func (wrapper *ResourceSliceWrapper) Obj() *resourceapi.ResourceSlice {
 	return &wrapper.ResourceSlice
 }
 
+// Devices sets the devices field of the inner object.
 func (wrapper *ResourceSliceWrapper) Devices(names ...string) *ResourceSliceWrapper {
 	for _, name := range names {
 		wrapper.Spec.Devices = append(wrapper.Spec.Devices, resourceapi.Device{Name: name})
@@ -1159,7 +1160,141 @@ func (wrapper *ResourceSliceWrapper) Devices(names ...string) *ResourceSliceWrap
 	return wrapper
 }
 
+// Device sets the devices field of the inner object.
 func (wrapper *ResourceSliceWrapper) Device(name string, attrs map[resourceapi.QualifiedName]resourceapi.DeviceAttribute) *ResourceSliceWrapper {
 	wrapper.Spec.Devices = append(wrapper.Spec.Devices, resourceapi.Device{Name: name, Basic: &resourceapi.BasicDevice{Attributes: attrs}})
 	return wrapper
 }
+
+// StorageClassWrapper wraps a StorageClass inside.
+type StorageClassWrapper struct{ storagev1.StorageClass }
+
+// MakeStorageClass creates a StorageClass wrapper.
+func MakeStorageClass() *StorageClassWrapper {
+	return &StorageClassWrapper{}
+}
+
+// Obj returns the inner StorageClass.
+func (s *StorageClassWrapper) Obj() *storagev1.StorageClass {
+	return &s.StorageClass
+}
+
+// Name sets `n` as the name of the inner StorageClass.
+func (s *StorageClassWrapper) Name(n string) *StorageClassWrapper {
+	s.SetName(n)
+	return s
+}
+
+// VolumeBindingMode sets mode as the mode of the inner StorageClass.
+func (s *StorageClassWrapper) VolumeBindingMode(mode storagev1.VolumeBindingMode) *StorageClassWrapper {
+	s.StorageClass.VolumeBindingMode = &mode
+	return s
+}
+
+// Provisoner sets p as the provisioner of the inner StorageClass.
+func (s *StorageClassWrapper) Provisioner(p string) *StorageClassWrapper {
+	s.StorageClass.Provisioner = p
+	return s
+}
+
+// AllowedTopologies sets `AllowedTopologies` of the inner StorageClass.
+func (s *StorageClassWrapper) AllowedTopologies(topologies []v1.TopologySelectorTerm) *StorageClassWrapper {
+	s.StorageClass.AllowedTopologies = topologies
+	return s
+}
+
+// Label sets a {k,v} pair to the inner StorageClass label.
+func (s *StorageClassWrapper) Label(k, v string) *StorageClassWrapper {
+	if s.ObjectMeta.Labels == nil {
+		s.ObjectMeta.Labels = make(map[string]string)
+	}
+	s.ObjectMeta.Labels[k] = v
+	return s
+}
+
+// CSINodeWrapper wraps a CSINode inside.
+type CSINodeWrapper struct{ storagev1.CSINode }
+
+// MakeCSINode creates a CSINode wrapper.
+func MakeCSINode() *CSINodeWrapper {
+	return &CSINodeWrapper{}
+}
+
+// Obj returns the inner CSINode.
+func (c *CSINodeWrapper) Obj() *storagev1.CSINode {
+	return &c.CSINode
+}
+
+// Name sets `n` as the name of the inner CSINode.
+func (c *CSINodeWrapper) Name(n string) *CSINodeWrapper {
+	c.SetName(n)
+	return c
+}
+
+// Annotation sets a {k,v} pair to the inner CSINode annotation.
+func (c *CSINodeWrapper) Annotation(key, value string) *CSINodeWrapper {
+	metav1.SetMetaDataAnnotation(&c.ObjectMeta, key, value)
+	return c
+}
+
+// Driver adds a driver to the inner CSINode.
+func (c *CSINodeWrapper) Driver(driver storagev1.CSINodeDriver) *CSINodeWrapper {
+	c.Spec.Drivers = append(c.Spec.Drivers, driver)
+	return c
+}
+
+// CSIDriverWrapper wraps a CSIDriver inside.
+type CSIDriverWrapper struct{ storagev1.CSIDriver }
+
+// MakeCSIDriver creates a CSIDriver wrapper.
+func MakeCSIDriver() *CSIDriverWrapper {
+	return &CSIDriverWrapper{}
+}
+
+// Obj returns the inner CSIDriver.
+func (c *CSIDriverWrapper) Obj() *storagev1.CSIDriver {
+	return &c.CSIDriver
+}
+
+// Name sets `n` as the name of the inner CSIDriver.
+func (c *CSIDriverWrapper) Name(n string) *CSIDriverWrapper {
+	c.SetName(n)
+	return c
+}
+
+// StorageCapacity sets the `StorageCapacity` of the inner CSIDriver.
+func (c *CSIDriverWrapper) StorageCapacity(storageCapacity *bool) *CSIDriverWrapper {
+	c.Spec.StorageCapacity = storageCapacity
+	return c
+}
+
+// CSIStorageCapacityWrapper wraps a CSIStorageCapacity inside.
+type CSIStorageCapacityWrapper struct{ storagev1.CSIStorageCapacity }
+
+// MakeCSIStorageCapacity creates a CSIStorageCapacity wrapper.
+func MakeCSIStorageCapacity() *CSIStorageCapacityWrapper {
+	return &CSIStorageCapacityWrapper{}
+}
+
+// Obj returns the inner CSIStorageCapacity.
+func (c *CSIStorageCapacityWrapper) Obj() *storagev1.CSIStorageCapacity {
+	return &c.CSIStorageCapacity
+}
+
+// Name sets `n` as the name of the inner CSIStorageCapacity.
+func (c *CSIStorageCapacityWrapper) Name(n string) *CSIStorageCapacityWrapper {
+	c.SetName(n)
+	return c
+}
+
+// StorageClassName sets the `StorageClassName` of the inner CSIStorageCapacity.
+func (c *CSIStorageCapacityWrapper) StorageClassName(name string) *CSIStorageCapacityWrapper {
+	c.CSIStorageCapacity.StorageClassName = name
+	return c
+}
+
+// Capacity sets the `Capacity` of the inner CSIStorageCapacity.
+func (c *CSIStorageCapacityWrapper) Capacity(capacity *resource.Quantity) *CSIStorageCapacityWrapper {
+	c.CSIStorageCapacity.Capacity = capacity
+	return c
+}
