@@ -19,7 +19,6 @@ package apimachinery
 import (
 	"context"
 	"fmt"
-	"math/rand"
 	"reflect"
 	"time"
 
@@ -30,18 +29,12 @@ import (
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
-	"k8s.io/apiserver/pkg/features"
 	"k8s.io/apiserver/pkg/storage/storagebackend"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/client-go/util/workqueue"
 	"k8s.io/kubernetes/test/e2e/framework"
 	admissionapi "k8s.io/pod-security-admission/api"
 )
 
-func shouldCheckRemainingItem() bool {
-	return utilfeature.DefaultFeatureGate.Enabled(features.RemainingItemCount)
-}
-
 const numberOfTotalResources = 400
 
 var _ = SIGDescribe("Servers with support for API chunking", func() {
@@ -76,7 +69,18 @@ var _ = SIGDescribe("Servers with support for API chunking", func() {
 		})
 	})
 
-	ginkgo.It("should return chunks of results for list calls", func(ctx context.Context) {
+	/*
+		Release: v1.29
+		Testname: API Chunking, server should return chunks of results for list calls
+		Description: Create a large number of PodTemplates. Attempt to retrieve the first chunk with limit set;
+		the server MUST return the chunk of the size not exceeding the limit with RemainingItems set in the response.
+		Attempt to retrieve the remaining items by providing the received continuation token and limit;
+		the server MUST return the remaining items in chunks of the size not exceeding the limit, with appropriately
+		set RemainingItems field in the response and with the ResourceVersion returned in the first response.
+		Attempt to list all objects at once without setting the limit; the server MUST return all items in a single
+		response.
+	*/
+	framework.ConformanceIt("should return chunks of results for list calls", func(ctx context.Context) {
 		ns := f.Namespace.Name
 		c := f.ClientSet
 		client := c.CoreV1().PodTemplates(ns)
@@ -86,7 +90,9 @@ var _ = SIGDescribe("Servers with support for API chunking", func() {
 			found := 0
 			var lastRV string
 			for {
-				opts.Limit = int64(rand.Int31n(numberOfTotalResources/10) + 1)
+				// With numberOfTotalResources=400, we want to ensure that both
+				// number of items per page and number of pages are non-trivial.
+				opts.Limit = 17
 				list, err := client.List(ctx, opts)
 				framework.ExpectNoError(err, "failed to list pod templates in namespace: %s, given limit: %d", ns, opts.Limit)
 				framework.Logf("Retrieved %d/%d results with rv %s and continue %s", len(list.Items), opts.Limit, list.ResourceVersion, list.Continue)
@@ -95,17 +101,15 @@ var _ = SIGDescribe("Servers with support for API chunking", func() {
 				if len(lastRV) == 0 {
 					lastRV = list.ResourceVersion
 				}
-				framework.ExpectEqual(list.ResourceVersion, lastRV)
-				if shouldCheckRemainingItem() {
-					if list.GetContinue() == "" {
-						gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
-					} else {
-						gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
-						gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items) + found).To(gomega.BeNumerically("==", numberOfTotalResources))
-					}
+				gomega.Expect(list.ResourceVersion).To(gomega.Equal(lastRV))
+				if list.GetContinue() == "" {
+					gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
+				} else {
+					gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
+					gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items) + found).To(gomega.BeNumerically("==", numberOfTotalResources))
 				}
 				for _, item := range list.Items {
-					framework.ExpectEqual(item.Name, fmt.Sprintf("template-%04d", found))
+					gomega.Expect(item.Name).To(gomega.Equal(fmt.Sprintf("template-%04d", found)))
 					found++
 				}
 				if len(list.Continue) == 0 {
@@ -123,7 +127,21 @@ var _ = SIGDescribe("Servers with support for API chunking", func() {
 		gomega.Expect(list.Items).To(gomega.HaveLen(numberOfTotalResources))
 	})
 
-	ginkgo.It("should support continue listing from the last key if the original version has been compacted away, though the list is inconsistent [Slow]", func(ctx context.Context) {
+	/*
+		Release: v1.29
+		Testname: API Chunking, server should support continue listing from the last key even if the original version has been compacted away
+		Description: Create a large number of PodTemplates. Attempt to retrieve the first chunk with limit set;
+		the server MUST return the chunk of the size not exceeding the limit with RemainingItems set in the response.
+		Attempt to retrieve the second page until the continuation token expires; the server MUST return a
+		continuation token for inconsistent list continuation.
+		Attempt to retrieve the second page with the received inconsistent list continuation token; the server
+		MUST return the number of items not exceeding the limit, a new continuation token and appropriately set
+		RemainingItems field in the response.
+		Attempt to retrieve the remaining pages by passing the received continuation token; the server
+		MUST return the remaining items in chunks of the size not exceeding the limit, with appropriately
+		set RemainingItems field in the response and with the ResourceVersion returned as part of the inconsistent list.
+	*/
+	framework.ConformanceIt("should support continue listing from the last key if the original version has been compacted away, though the list is inconsistent", f.WithSlow(), func(ctx context.Context) {
 		ns := f.Namespace.Name
 		c := f.ClientSet
 		client := c.CoreV1().PodTemplates(ns)
@@ -136,13 +154,11 @@ var _ = SIGDescribe("Servers with support for API chunking", func() {
 		framework.ExpectNoError(err, "failed to list pod templates in namespace: %s, given limit: %d", ns, opts.Limit)
 		firstToken := list.Continue
 		firstRV := list.ResourceVersion
-		if shouldCheckRemainingItem() {
-			if list.GetContinue() == "" {
-				gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
-			} else {
-				gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
-				gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items)).To(gomega.BeNumerically("==", numberOfTotalResources))
-			}
+		if list.GetContinue() == "" {
+			gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
+		} else {
+			gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
+			gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items)).To(gomega.BeNumerically("==", numberOfTotalResources))
 		}
 		framework.Logf("Retrieved %d/%d results with rv %s and continue %s", len(list.Items), opts.Limit, list.ResourceVersion, firstToken)
 
@@ -175,20 +191,18 @@ var _ = SIGDescribe("Servers with support for API chunking", func() {
 		opts.Continue = inconsistentToken
 		list, err = client.List(ctx, opts)
 		framework.ExpectNoError(err, "failed to list pod templates in namespace: %s, given inconsistent continue token %s and limit: %d", ns, opts.Continue, opts.Limit)
-		framework.ExpectNotEqual(list.ResourceVersion, firstRV)
+		gomega.Expect(list.ResourceVersion).ToNot(gomega.Equal(firstRV))
 		gomega.Expect(len(list.Items)).To(gomega.BeNumerically("==", opts.Limit))
 		found := int(oneTenth)
 
-		if shouldCheckRemainingItem() {
-			if list.GetContinue() == "" {
-				gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
-			} else {
-				gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
-				gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items) + found).To(gomega.BeNumerically("==", numberOfTotalResources))
-			}
+		if list.GetContinue() == "" {
+			gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
+		} else {
+			gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
+			gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items) + found).To(gomega.BeNumerically("==", numberOfTotalResources))
 		}
 		for _, item := range list.Items {
-			framework.ExpectEqual(item.Name, fmt.Sprintf("template-%04d", found))
+			gomega.Expect(item.Name).To(gomega.Equal(fmt.Sprintf("template-%04d", found)))
 			found++
 		}
 
@@ -198,19 +212,17 @@ var _ = SIGDescribe("Servers with support for API chunking", func() {
 		for {
 			list, err := client.List(ctx, opts)
 			framework.ExpectNoError(err, "failed to list pod templates in namespace: %s, given limit: %d", ns, opts.Limit)
-			if shouldCheckRemainingItem() {
-				if list.GetContinue() == "" {
-					gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
-				} else {
-					gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
-					gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items) + found).To(gomega.BeNumerically("==", numberOfTotalResources))
-				}
+			if list.GetContinue() == "" {
+				gomega.Expect(list.GetRemainingItemCount()).To(gomega.BeNil())
+			} else {
+				gomega.Expect(list.GetRemainingItemCount()).ToNot(gomega.BeNil())
+				gomega.Expect(int(*list.GetRemainingItemCount()) + len(list.Items) + found).To(gomega.BeNumerically("==", numberOfTotalResources))
 			}
 			framework.Logf("Retrieved %d/%d results with rv %s and continue %s", len(list.Items), opts.Limit, list.ResourceVersion, list.Continue)
 			gomega.Expect(len(list.Items)).To(gomega.BeNumerically("<=", opts.Limit))
-			framework.ExpectEqual(list.ResourceVersion, lastRV)
+			gomega.Expect(list.ResourceVersion).To(gomega.Equal(lastRV))
 			for _, item := range list.Items {
-				framework.ExpectEqual(item.Name, fmt.Sprintf("template-%04d", found))
+				gomega.Expect(item.Name).To(gomega.Equal(fmt.Sprintf("template-%04d", found)))
 				found++
 			}
 			if len(list.Continue) == 0 {
