@@ -48,6 +48,7 @@ import (
 	e2eskipper "k8s.io/kubernetes/test/e2e/framework/skipper"
 	e2etestfiles "k8s.io/kubernetes/test/e2e/framework/testfiles"
 	e2etestingmanifests "k8s.io/kubernetes/test/e2e/testing-manifests"
+	"k8s.io/kubernetes/test/e2e_node/criproxy"
 	"k8s.io/kubernetes/test/e2e_node/services"
 	e2enodetestingmanifests "k8s.io/kubernetes/test/e2e_node/testing-manifests"
 	system "k8s.io/system-validators/validators"
@@ -69,7 +70,8 @@ import (
 )
 
 var (
-	e2es *services.E2EServices
+	e2eCriProxy *criproxy.RemoteRuntime
+	e2es        *services.E2EServices
 	// featureGates is a map of feature names to bools that enable or disable alpha/experimental features.
 	featureGates map[string]bool
 	// serviceFeatureGates is a map of feature names to bools that enable or
@@ -109,6 +111,7 @@ func registerNodeFlags(flags *flag.FlagSet) {
 	flags.Var(cliflag.NewMapStringBool(&featureGates), "feature-gates", "A set of key=value pairs that describe feature gates for alpha/experimental features.")
 	flags.Var(cliflag.NewMapStringBool(&serviceFeatureGates), "service-feature-gates", "A set of key=value pairs that describe feature gates for alpha/experimental features for API service.")
 	flags.BoolVar(&framework.TestContext.StandaloneMode, "standalone-mode", false, "If true, starts kubelet in standalone mode.")
+	flags.BoolVar(&framework.TestContext.CriProxyEnabled, "cri-proxy-enabled", false, "If true, enable CRI API proxy for failure injection.")
 }
 
 func init() {
@@ -190,8 +193,12 @@ func TestE2eNode(t *testing.T) {
 				klog.Exitf("chroot %q failed: %v", rootfs, err)
 			}
 		}
-		if _, err := system.ValidateSpec(*spec, "remote"); len(err) != 0 {
-			klog.Exitf("system validation failed: %v", err)
+		warns, errs := system.ValidateSpec(*spec, "remote")
+		if len(warns) != 0 {
+			klog.Warningf("system validation warns: %v", warns)
+		}
+		if len(errs) != 0 {
+			klog.Exitf("system validation failed: %v", errs)
 		}
 		return
 	}
@@ -233,7 +240,7 @@ var _ = ginkgo.SynchronizedBeforeSuite(func(ctx context.Context) []byte {
 	if framework.TestContext.PrepullImages {
 		klog.Infof("Pre-pulling images so that they are cached for the tests.")
 		updateImageAllowList(ctx)
-		err := PrePullAllImages()
+		err := PrePullAllImages(ctx)
 		gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
 	}
 
@@ -242,6 +249,22 @@ var _ = ginkgo.SynchronizedBeforeSuite(func(ctx context.Context) []byte {
 	// We should mask locksmithd when provisioning the machine.
 	maskLocksmithdOnCoreos()
 
+	if framework.TestContext.CriProxyEnabled {
+		framework.Logf("Start cri proxy")
+		rs, is, err := getCRIClient()
+		framework.ExpectNoError(err)
+
+		e2eCriProxy = criproxy.NewRemoteRuntimeProxy(rs, is)
+		endpoint, err := criproxy.GenerateEndpoint()
+		framework.ExpectNoError(err)
+
+		err = e2eCriProxy.Start(endpoint)
+		framework.ExpectNoError(err)
+
+		framework.TestContext.ContainerRuntimeEndpoint = endpoint
+		framework.TestContext.ImageServiceEndpoint = endpoint
+	}
+
 	if *startServices {
 		// If the services are expected to stop after test, they should monitor the test process.
 		// If the services are expected to keep running after test, they should not monitor the test process.
@@ -285,6 +308,11 @@ var _ = ginkgo.SynchronizedAfterSuite(func() {}, func() {
 		}
 	}
 
+	if e2eCriProxy != nil {
+		framework.Logf("Stopping cri proxy service...")
+		e2eCriProxy.Stop()
+	}
+
 	klog.Infof("Tests Finished")
 })
 
