@@ -231,10 +231,7 @@ var _ = SIGDescribe("MirrorPod", func() {
 			}
 
 			ginkgo.By("Stopping the NFS server")
-			stopNfsServer(f, nfsServerPod)
-
-			ginkgo.By("Waiting for NFS server to stop...")
-			time.Sleep(30 * time.Second)
+			e2evolume.StopNFSServer(f, nfsServerPod)
 
 			ginkgo.By(fmt.Sprintf("Deleting the static nfs test pod: %s", staticPodName))
 			err = deleteStaticPod(podPath, staticPodName, ns)
@@ -243,15 +240,15 @@ var _ = SIGDescribe("MirrorPod", func() {
 			// Wait 5 mins for syncTerminatedPod to fail. We expect that the pod volume should not be cleaned up because the NFS server is down.
 			gomega.Consistently(func() bool {
 				return podVolumeDirectoryExists(types.UID(hash))
-			}, 5*time.Minute, 10*time.Second).Should(gomega.BeTrue(), "pod volume should exist while nfs server is stopped")
+			}, 5*time.Minute, 10*time.Second).Should(gomega.BeTrueBecause("pod volume should exist while nfs server is stopped"))
 
 			ginkgo.By("Start the NFS server")
-			restartNfsServer(f, nfsServerPod)
+			e2evolume.RestartNFSServer(f, nfsServerPod)
 
 			ginkgo.By("Waiting for the pod volume to deleted after the NFS server is started")
 			gomega.Eventually(func() bool {
 				return podVolumeDirectoryExists(types.UID(hash))
-			}, 5*time.Minute, 10*time.Second).Should(gomega.BeFalse(), "pod volume should be deleted after nfs server is started")
+			}, 5*time.Minute, 10*time.Second).Should(gomega.BeFalseBecause("pod volume should be deleted after nfs server is started"))
 
 			// Create the static pod again with the same config and expect it to start running
 			err = createStaticPodUsingNfs(nfsServerHost, node, "sleep 999999", podPath, staticPodName, ns)
@@ -289,25 +286,6 @@ func podVolumeDirectoryExists(uid types.UID) bool {
 	return podVolumeDirectoryExists
 }
 
-// Restart the passed-in nfs-server by issuing a `/usr/sbin/rpc.nfsd 1` command in the
-// pod's (only) container. This command changes the number of nfs server threads from
-// (presumably) zero back to 1, and therefore allows nfs to open connections again.
-func restartNfsServer(f *framework.Framework, serverPod *v1.Pod) {
-	const startcmd = "/usr/sbin/rpc.nfsd 1"
-	_, _, err := e2evolume.PodExec(f, serverPod, startcmd)
-	framework.ExpectNoError(err)
-
-}
-
-// Stop the passed-in nfs-server by issuing a `/usr/sbin/rpc.nfsd 0` command in the
-// pod's (only) container. This command changes the number of nfs server threads to 0,
-// thus closing all open nfs connections.
-func stopNfsServer(f *framework.Framework, serverPod *v1.Pod) {
-	const stopcmd = "/usr/sbin/rpc.nfsd 0"
-	_, _, err := e2evolume.PodExec(f, serverPod, stopcmd)
-	framework.ExpectNoError(err)
-}
-
 func createStaticPodUsingNfs(nfsIP string, nodeName string, cmd string, dir string, name string, ns string) error {
 	ginkgo.By("create pod using nfs volume")
 
